#pragma once

#pragma comment(linker, "/NODEFAULTLIB") 
#pragma comment( lib, "kernel32" )
#pragma comment( lib, "gdi32" )  
#pragma comment(linker, "/ENTRY:\"DllEntryPoint\"") // define entry point cause no C Lib

#define WIN32_LEAN_AND_MEAN
#define INITGUID
#include <windows.h>
#include <guiddef.h>
#include <ddraw.h>
#include <dvp.h>
#include "ddrawi.h"
#include "my_no_crt.h"

struct WRAP
{
    void* vtable;
	long  magic;
	union
	{
		IUnknown*     iunknown;
		IClassFactory* classfactory;
		IDirectDraw*  dd1;
		IDirectDraw2* dd2;
		IDirectDraw4* dd4;
		IDirectDraw7* dd7;
		IDirectDrawSurface*  dds1;
		IDirectDrawSurface2* dds2;
		IDirectDrawSurface3* dds3;
		IDirectDrawSurface4* dds4;
		IDirectDrawSurface7* dds7;
	};
	GUID  iid; 
	volatile long ref;
	long  lparam; // if surface then this is dd_wrapper 
	WRAP* next;
	WRAP* prev;
};

struct EnumStruct
{
	WRAP* ddraw;
	LPVOID lpOriginalCallback;
	LPVOID lpOriginalContext;
};


// cfg.cpp
struct CONFIG
{
	DWORD Affinity;
	DWORD BltMirror;
	DWORD ColorFix;
	DWORD DpiAware;
	DWORD ForceHEL;
	DWORD FakeVsync;
	DWORD FullFlip;
	VOID Init();
	CHAR szDDrawPath[MAX_PATH];
};
#define AUTO_CONFIG -1
BOOL TestVsync( WRAP* dd );
DWORD TestBltMirror( WRAP* dd, DWORD dwDescSize );
DWORD TestFlip( WRAP* dd, DWORD dwDescSize );
extern CONFIG Config;


// fixes.cpp
HRESULT FlipProc( WRAP* wrap, LPDIRECTDRAWSURFACE lpDDSurfaceTargetOverride, DWORD dwFlags );
HRESULT BltProc( WRAP* wrap, LPRECT lpDestRect, LPDIRECTDRAWSURFACE lpDDSrcSurface, LPRECT lpSrcRect, DWORD dwFlags, LPDDBLTFX lpDDBltFx );
void DisableHighDPIScaling();
HRESULT WaitForVerticalBlankProc( WRAP* wrap, DWORD dwFlags, HANDLE hEvent );

// dd.cpp
struct IDirectDraw1_vtable;
struct IDirectDraw2_vtable;
struct IDirectDraw4_vtable;
struct IDirectDraw7_vtable;
extern IDirectDraw1_vtable dd1_vtable;
extern IDirectDraw2_vtable dd2_vtable;
extern IDirectDraw4_vtable dd4_vtable;
extern IDirectDraw7_vtable dd7_vtable;

// dds.cpp
struct IDirectDrawSurface1_vtable;
struct IDirectDrawSurface2_vtable;
struct IDirectDrawSurface3_vtable;
struct IDirectDrawSurface4_vtable;
struct IDirectDrawSurface7_vtable;
extern IDirectDrawSurface1_vtable dds1_vtable;
extern IDirectDrawSurface2_vtable dds2_vtable;
extern IDirectDrawSurface3_vtable dds3_vtable;
extern IDirectDrawSurface4_vtable dds4_vtable;
extern IDirectDrawSurface7_vtable dds7_vtable;

// unknwn.cpp
struct IUnknown_vtable;
struct IClassFactory_vtable;
extern IUnknown_vtable unkn_vtable;
extern IClassFactory_vtable cf_vtable;

// dllmain.cpp
extern HMODULE g_hDll;

// wrapper.cpp
extern CRITICAL_SECTION WrapCriticalSection;
void DeleteWrap( WRAP* wrapper );
ULONG AddRefWrap( WRAP* wrapper );
ULONG ReleaseWrap( WRAP* wrapper );
bool GetWrap( void** ppv, long lparam, const GUID& riid );
IUnknown*            GetInterfacePtr( IUnknown*            wrapper );
IDirectDraw*         GetInterfacePtr( IDirectDraw*         wrapper );
IDirectDraw2*        GetInterfacePtr( IDirectDraw2*        wrapper );
IDirectDraw4*        GetInterfacePtr( IDirectDraw4*        wrapper );
IDirectDraw7*        GetInterfacePtr( IDirectDraw7*        wrapper );
IDirectDrawSurface*  GetInterfacePtr( IDirectDrawSurface*  wrapper );
IDirectDrawSurface2* GetInterfacePtr( IDirectDrawSurface2* wrapper );
IDirectDrawSurface3* GetInterfacePtr( IDirectDrawSurface3* wrapper );
IDirectDrawSurface4* GetInterfacePtr( IDirectDrawSurface4* wrapper );
IDirectDrawSurface7* GetInterfacePtr( IDirectDrawSurface7* wrapper );
HRESULT __stdcall EnumSurfaces1Callback( LPDIRECTDRAWSURFACE  lpDDSurface, LPDDSURFACEDESC  lpDDSurfaceDesc, LPVOID lpContext );
HRESULT __stdcall EnumSurfaces4Callback( LPDIRECTDRAWSURFACE4 lpDDSurface, LPDDSURFACEDESC2 lpDDSurfaceDesc, LPVOID lpContext );
HRESULT __stdcall EnumSurfaces7Callback( LPDIRECTDRAWSURFACE7 lpDDSurface, LPDDSURFACEDESC2 lpDDSurfaceDesc, LPVOID lpContext );

